@@ -11,7 +11,7 @@ import pysal
 from Contiguity import buildContiguity
 from Distance import knnW, Kernel, DistanceBand
 from util import get_ids, get_points_array_from_shapefile, min_threshold_distance
-
+import numpy as np
 
 def queen_from_shapefile(shapefile, idVariable=None, sparse=False):
     """
@@ -65,9 +65,11 @@ def queen_from_shapefile(shapefile, idVariable=None, sparse=False):
         ids = None
     w = buildContiguity(shp, criterion='queen', ids=ids)
     shp.close()
+    w.set_shapefile(shapefile, idVariable)
 
     if sparse:
         w = pysal.weights.WSP(w.sparse, id_order=ids)
+
     return w
 
 
@@ -117,8 +119,11 @@ def rook_from_shapefile(shapefile, idVariable=None, sparse=False):
         ids = None
     w = buildContiguity(shp, criterion='rook', ids=ids)
     shp.close()
+    w.set_shapefile(shapefile, idVariable)
     if sparse:
         w = pysal.weights.WSP(w.sparse, id_order=ids)
+
+
     return w
 
 
@@ -190,28 +195,28 @@ def knnW_from_array(array, k=2, p=2, ids=None, radius=None):
     >>> data=np.hstack([x,y])
     >>> wnn2=knnW_from_array(data,k=2)
     >>> wnn4=knnW_from_array(data,k=4)
-    >>> wnn4.neighbors[0]
-    [1, 5, 6, 2]
-    >>> wnn4.neighbors[5]
-    [0, 6, 10, 1]
-    >>> wnn2.neighbors[0]
-    [1, 5]
-    >>> wnn2.neighbors[5]
-    [0, 6]
+    >>> set([1, 5, 6, 2]) == set(wnn4.neighbors[0])
+    True
+    >>> set([0, 1, 10, 6]) == set(wnn4.neighbors[5])
+    True
+    >>> set([1, 5]) == set(wnn2.neighbors[0])
+    True
+    >>> set([0,6]) == set(wnn2.neighbors[5])
+    True
     >>> wnn2.pct_nonzero
     0.080000000000000002
     >>> wnn4.pct_nonzero
     0.16
     >>> wnn4=knnW_from_array(data,k=4)
-    >>> wnn4.neighbors[0]
-    [1, 5, 6, 2]
+    >>> set([ 1,5,6,2]) == set(wnn4.neighbors[0])
+    True
     >>> wnn4=knnW_from_array(data,k=4)
     >>> wnn3e=knnW(data,p=2,k=3)
-    >>> wnn3e.neighbors[0]
-    [1, 5, 6]
+    >>> set([1,5,6]) == set(wnn3e.neighbors[0])
+    True
     >>> wnn3m=knnW(data,p=1,k=3)
-    >>> wnn3m.neighbors[0]
-    [1, 5, 2]
+    >>> set([1,5,2]) == set(wnn3m.neighbors[0])
+    True
 
 
     Notes
@@ -265,13 +270,24 @@ def knnW_from_shapefile(shapefile, k=2, p=2, idVariable=None, radius=None):
     >>> wc=knnW_from_shapefile(pysal.examples.get_path("columbus.shp"))
     >>> wc.pct_nonzero
     0.040816326530612242
-    >>> wc3=knnW_from_shapefile(pysal.examples.get_path("columbus.shp"),k=3,idVariable="POLYID")
-    >>> wc3.weights[1]
-    [1, 1, 1]
-    >>> wc3.neighbors[1]
-    [3, 2, 4]
-    >>> wc.neighbors[0]
-    [2, 1]
+    >>> set([2,1]) == set(wc.neighbors[0])
+    True
+    >>> wc3=pysal.knnW_from_shapefile(pysal.examples.get_path("columbus.shp"),k=3)
+    >>> set(wc3.neighbors[0]) == set([2,1,3])
+    True
+    >>> set(wc3.neighbors[2]) == set([4,3,0])
+    True
+
+    1 offset rather than 0 offset
+
+    >>> wc3_1=knnW_from_shapefile(pysal.examples.get_path("columbus.shp"),k=3,idVariable="POLYID")
+    >>> set([4,3,2]) == set(wc3_1.neighbors[1])
+    True
+    >>> wc3_1.weights[2]
+    [1.0, 1.0, 1.0]
+    >>> set([4,1,8]) == set(wc3_1.neighbors[2])
+    True
+    
 
     Point shapefile
 
@@ -338,6 +354,8 @@ def threshold_binaryW_from_array(array, threshold, p=2, radius=None):
     --------
     >>> points=[(10, 10), (20, 10), (40, 10), (15, 20), (30, 20), (30, 30)]
     >>> w=threshold_binaryW_from_array(points,threshold=11.2)
+    WARNING: there is one disconnected observation (no neighbors)
+    Island id:  [2]
     >>> w.weights
     {0: [1, 1], 1: [1, 1], 2: [], 3: [1, 1], 4: [1], 5: [1]}
     >>> w.neighbors
@@ -441,12 +459,16 @@ def threshold_continuousW_from_array(array, threshold, p=2,
 
     >>> points=[(10, 10), (20, 10), (40, 10), (15, 20), (30, 20), (30, 30)]
     >>> wid=threshold_continuousW_from_array(points,11.2)
+    WARNING: there is one disconnected observation (no neighbors)
+    Island id:  [2]
     >>> wid.weights[0]
     [0.10000000000000001, 0.089442719099991588]
 
     gravity weights
 
     >>> wid2=threshold_continuousW_from_array(points,11.2,alpha=-2.0)
+    WARNING: there is one disconnected observation (no neighbors)
+    Island id:  [2]
     >>> wid2.weights[0]
     [0.01, 0.0079999999999999984]
 
@@ -511,13 +533,18 @@ def threshold_continuousW_from_shapefile(shapefile, threshold, p=2,
         data = pysal.cg.KDTree(data, distance_metric='Arc', radius=radius)
     if idVariable:
         ids = get_ids(shapefile, idVariable)
-        return DistanceBand(data, threshold=threshold, p=p, alpha=alpha, binary=False, ids=ids)
-    return threshold_continuousW_from_array(data, threshold, p=p, alpha=alpha)
+        w = DistanceBand(data, threshold=threshold, p=p, alpha=alpha, binary=False, ids=ids)
+    else:
+        w =  threshold_continuousW_from_array(data, threshold, p=p, alpha=alpha)
+    w.set_shapefile(shapefile,idVariable)
+    return w
+
 
 # Kernel Weights
 
 
-def kernelW(points, k=2, function='triangular', fixed=True, radius=None):
+def kernelW(points, k=2, function='triangular', fixed=True,
+        radius=None, diagonal=False):
     """
     Kernel based weights
 
@@ -588,6 +615,10 @@ def kernelW(points, k=2, function='triangular', fixed=True, radius=None):
                    bandwidth is adaptive across observations.
     radius     : If supplied arc_distances will be calculated
                  based on the given radius. p will be ignored.
+    diagonal   : boolean
+                 If true, set diagonal weights = 1.0, if false (default)
+                 diagonal weights are set to value according to kernel
+                 function
 
     Returns
     -------
@@ -623,13 +654,26 @@ def kernelW(points, k=2, function='triangular', fixed=True, radius=None):
            [ 22.36068201],
            [ 22.36068201],
            [ 22.36068201]])
+
+    Diagonals to 1.0
+
+    >>> kq = kernelW(points,function='gaussian')
+    >>> kq.weights
+    {0: [0.3989422804014327, 0.35206533556593145, 0.3412334260702758], 1: [0.35206533556593145, 0.3989422804014327, 0.2419707487162134, 0.3412334260702758, 0.31069657591175387], 2: [0.2419707487162134, 0.3989422804014327, 0.31069657591175387], 3: [0.3412334260702758, 0.3412334260702758, 0.3989422804014327, 0.3011374490937829, 0.26575287272131043], 4: [0.31069657591175387, 0.31069657591175387, 0.3011374490937829, 0.3989422804014327, 0.35206533556593145], 5: [0.26575287272131043, 0.35206533556593145, 0.3989422804014327]}
+    >>> kqd = kernelW(points, function='gaussian', diagonal=True)
+    >>> kqd.weights
+    {0: [1.0, 0.35206533556593145, 0.3412334260702758], 1: [0.35206533556593145, 1.0, 0.2419707487162134, 0.3412334260702758, 0.31069657591175387], 2: [0.2419707487162134, 1.0, 0.31069657591175387], 3: [0.3412334260702758, 0.3412334260702758, 1.0, 0.3011374490937829, 0.26575287272131043], 4: [0.31069657591175387, 0.31069657591175387, 0.3011374490937829, 1.0, 0.35206533556593145], 5: [0.26575287272131043, 0.35206533556593145, 1.0]}
+
+
     """
     if radius is not None:
         points = pysal.cg.KDTree(points, distance_metric='Arc', radius=radius)
-    return Kernel(points, function=function, k=k, fixed=fixed)
+    return Kernel(points, function=function, k=k, fixed=fixed,
+            diagonal=diagonal)
 
 
-def kernelW_from_shapefile(shapefile, k=2, function='triangular', idVariable=None, fixed=True, radius=None):
+def kernelW_from_shapefile(shapefile, k=2, function='triangular',
+        idVariable=None, fixed=True, radius=None, diagonal=False):
     """
     Kernel based weights
 
@@ -701,6 +745,10 @@ def kernelW_from_shapefile(shapefile, k=2, function='triangular', idVariable=Non
                    bandwidth is adaptive across observations.
     radius     : If supplied arc_distances will be calculated
                  based on the given radius. p will be ignored.
+    diagonal   : boolean
+                 If true, set diagonal weights = 1.0, if false (default)
+                 diagonal weights are set to value according to kernel
+                 function
 
 
     Returns
@@ -711,14 +759,18 @@ def kernelW_from_shapefile(shapefile, k=2, function='triangular', idVariable=Non
 
     Examples
     --------
-    >>> kw = kernelW_from_shapefile(pysal.examples.get_path("columbus.shp"),idVariable='POLYID')
-    >>> kw.weights[1]
-    [0.2052478782400463, 0.007078773148450623, 1.0, 0.23051223027663237]
-    >>> kw.bandwidth[:3]
-    array([[ 0.75333961],
-           [ 0.75333961],
-           [ 0.75333961]])
+    >>> kw = pysal.kernelW_from_shapefile(pysal.examples.get_path("columbus.shp"),idVariable='POLYID', function = 'gaussian')
 
+    >>> kwd = pysal.kernelW_from_shapefile(pysal.examples.get_path("columbus.shp"),idVariable='POLYID', function = 'gaussian', diagonal = True)
+    >>> kw.neighbors[1]
+    [2, 4, 1, 3]
+    >>> kwd.neighbors[1]
+    [2, 4, 1, 3]
+    >>> kw.weights[1]
+    [0.29090631630909874, 0.2436835517263174, 0.3989422804014327, 0.29671172124745776]
+    >>> kwd.weights[1]
+    [0.29090631630909874, 0.2436835517263174, 1.0, 0.29671172124745776]
+    
 
     Notes
     -----
@@ -734,11 +786,14 @@ def kernelW_from_shapefile(shapefile, k=2, function='triangular', idVariable=Non
         points = pysal.cg.KDTree(points, distance_metric='Arc', radius=radius)
     if idVariable:
         ids = get_ids(shapefile, idVariable)
-        return Kernel(points, function=function, k=k, ids=ids, fixed=fixed)
-    return kernelW(points, k=k, function=function, fixed=fixed)
+        return Kernel(points, function=function, k=k, ids=ids, fixed=fixed,
+                diagonal = diagonal)
+    return kernelW(points, k=k, function=function, fixed=fixed,
+            diagonal=diagonal)
 
 
-def adaptive_kernelW(points, bandwidths=None, k=2, function='triangular', radius=None):
+def adaptive_kernelW(points, bandwidths=None, k=2, function='triangular',
+        radius=None, diagonal=False):
     """
     Kernel weights with adaptive bandwidths
 
@@ -798,7 +853,10 @@ def adaptive_kernelW(points, bandwidths=None, k=2, function='triangular', radius
 
     radius     : If supplied arc_distances will be calculated
                  based on the given radius. p will be ignored.
-
+    diagonal   : boolean
+                 If true, set diagonal weights = 1.0, if false (default)
+                 diagonal weights are set to value according to kernel
+                 function
     Returns
     -------
 
@@ -853,14 +911,31 @@ def adaptive_kernelW(points, bandwidths=None, k=2, function='triangular', radius
            [ 11.18034101],
            [ 14.14213704],
            [ 18.02775818]])
+
+    with diagonal
+
+    >>> kweag = pysal.adaptive_kernelW(points, function='gaussian')
+    >>> kweagd = pysal.adaptive_kernelW(points, function='gaussian', diagonal=True)
+    >>> kweag.neighbors[0]
+    [0, 1, 3]
+    >>> kweagd.neighbors[0]
+    [0, 1, 3]
+    >>> kweag.weights[0]
+    [0.3989422804014327, 0.2674190291577696, 0.2419707487162134]
+    >>> kweagd.weights[0]
+    [1.0, 0.2674190291577696, 0.2419707487162134]
+
+
     """
     if radius is not None:
         points = pysal.cg.KDTree(points, distance_metric='Arc', radius=radius)
-    return Kernel(points, bandwidth=bandwidths, fixed=False, k=k, function=function)
+    return Kernel(points, bandwidth=bandwidths, fixed=False, k=k,
+            function=function, diagonal=diagonal)
 
 
 def adaptive_kernelW_from_shapefile(shapefile, bandwidths=None, k=2, function='triangular',
-                                    idVariable=None, radius=None):
+                                    idVariable=None, radius=None,
+                                    diagonal = False):
     """
     Kernel weights with adaptive bandwidths
 
@@ -919,6 +994,10 @@ def adaptive_kernelW_from_shapefile(shapefile, bandwidths=None, k=2, function='t
                    name of a column in the shapefile's DBF to use for ids
     radius     : If supplied arc_distances will be calculated
                  based on the given radius. p will be ignored.
+    diagonal   : boolean
+                 If true, set diagonal weights = 1.0, if false (default)
+                 diagonal weights are set to value according to kernel
+                 function
 
     Returns
     -------
@@ -929,13 +1008,17 @@ def adaptive_kernelW_from_shapefile(shapefile, bandwidths=None, k=2, function='t
 
     Examples
     --------
-    >>> kwa = adaptive_kernelW_from_shapefile(pysal.examples.get_path("columbus.shp"))
+    >>> kwa = pysal.adaptive_kernelW_from_shapefile(pysal.examples.get_path("columbus.shp"), function='gaussian')
+    >>> kwad = pysal.adaptive_kernelW_from_shapefile(pysal.examples.get_path("columbus.shp"), function='gaussian', diagonal=True)
+    >>> kwa.neighbors[0]
+    [0, 2, 1]
+    >>> kwad.neighbors[0]
+    [0, 2, 1]
     >>> kwa.weights[0]
-    [1.0, 0.03178906767736345, 9.99999900663795e-08]
-    >>> kwa.bandwidth[:3]
-    array([[ 0.59871832],
-           [ 0.59871832],
-           [ 0.56095647]])
+    [0.3989422804014327, 0.24966013701844503, 0.2419707487162134]
+    >>> kwad.weights[0]
+    [1.0, 0.24966013701844503, 0.2419707487162134]
+    >>>
 
     Notes
     -----
@@ -950,8 +1033,10 @@ def adaptive_kernelW_from_shapefile(shapefile, bandwidths=None, k=2, function='t
         points = pysal.cg.KDTree(points, distance_metric='Arc', radius=radius)
     if idVariable:
         ids = get_ids(shapefile, idVariable)
-        return Kernel(points, bandwidth=bandwidths, fixed=False, k=k, function=function, ids=ids)
-    return adaptive_kernelW(points, bandwidths=bandwidths, k=k, function=function)
+        return Kernel(points, bandwidth=bandwidths, fixed=False, k=k,
+                function=function, ids=ids, diagonal=diagonal)
+    return adaptive_kernelW(points, bandwidths=bandwidths, k=k,
+            function=function, diagonal=diagonal)
 
 
 def min_threshold_dist_from_shapefile(shapefile, radius=None, p=2):
@@ -1020,6 +1105,11 @@ def build_lattice_shapefile(nrows, ncols, outFileName):
     if not outFileName.endswith('.shp'):
         raise ValueError("outFileName must end with .shp")
     o = pysal.open(outFileName, 'w')
+    dbf_name = outFileName.split(".")[0] + ".dbf"
+    d = pysal.open(dbf_name, 'w')
+    d.header = [ 'ID' ]
+    d.field_spec = [ ('N', 8, 0) ]
+    c = 0
     for i in xrange(nrows):
         for j in xrange(ncols):
             ll = i, j
@@ -1027,5 +1117,21 @@ def build_lattice_shapefile(nrows, ncols, outFileName):
             ur = i + 1, j + 1
             lr = i + 1, j
             o.write(pysal.cg.Polygon([ll, ul, ur, lr, ll]))
+            d.write([c])
+            c += 1
+    d.close()
     o.close()
 
+def _test():
+    import doctest
+    # the following line could be used to define an alternative to the '<BLANKLINE>' flag
+    #doctest.BLANKLINE_MARKER = 'something better than <BLANKLINE>'
+    start_suppress = np.get_printoptions()['suppress']
+    np.set_printoptions(suppress=True)
+    doctest.testmod()
+    np.set_printoptions(suppress=start_suppress)    
+
+if __name__ == '__main__':
+    _test()
+
+
